%
% Basic interior point algorithm implementation
%

function [x,err,steps,outersteps,s_b,y_b,m_b] = ip(x0, f, g, f_x, g_x, m, eps, delta_b, maxsteps, bfgsmethod, lbfgssteps,yinit,sinit,minit)

max_m_b = 0;
n = size(x0,1);

switch bfgsmethod
	case 'bfgs'
		method=1;
	case 'lbfgs'
		method=2;
		max_m_b = lbfgssteps;
		
		if exist('yinit','var') == 0 || exist('sinit','var') == 0 || exist('minit','var') == 0
			m_b = 0;
			s_b = zeros(n,max_m_b);
			y_b = zeros(n,max_m_b);
		else
			m_b = minit;
			s_b = sinit;
			y_b = yinit;
		end
		count_b = 0;
	otherwise
		fprintf('unknown bfgs method specified. expected ''bfgs'' or ''lbfgs''\n');
		err=1;
		x=0;
		steps=0;
		outersteps=0;
		s_b=0;
		y_b=0;
		m_b=0;
		return;
end

s = ones(m,1);
z = ones(m,1);
mu = 0.1;
minustau = 0.005;
outereps = 1.0;
outersteps = 0;
steps = 0;
Bcomputed = 0;

x=x0;

err=0;

p_x=zeros(n,1);
p_s=zeros(m,1);
p_z=zeros(m,1);

	
while outereps > eps && steps < maxsteps
	tau = 1-minustau;
	innereps = 1.0;
	while innereps > mu && innereps > eps && steps < maxsteps
				steps = steps+1;

		Z = diag(z);
		S = diag(s);
		
	
		sig = S\Z;
  		gx_temp = g_x(x);
  		g_temp = g(x);

		b = -f_x(x)' + gx_temp'*(z - sig*g_temp + mu*S\ones(m,1));
	
        switch method
			case 1
				if Bcomputed == 0
					B=eye(n)*delta_b;
					Bcomputed=1;
				else
					theta = 1.0;
					Bsk=B*sk;
					skBsk = sk'*Bsk;
					if sk'*yk < 0.2*skBsk
						theta = (0.8*skBsk)/(skBsk - sk'*yk);
					end
					rk=theta*yk + (1-theta)*Bsk;
					B = B - (Bsk*Bsk')/(skBsk) + (rk*rk')/(sk'*rk);
				end
				
				A = B + gx_temp'*sig*gx_temp;
				[p_x,err] = choleskysolve(A,b);
				if err==1
					fprintf('Indefinite Hessian...\n');
					break;
				end
				p_s = g_temp - s + gx_temp*p_x;
				p_z = -z + mu*inv(S)*ones(m,1) - sig*p_s;

			case 2
			
				
				% Spezialfall: Boxed Constraints erzeugen hier Diagonalmatrix... leicht zu invertieren
				tinv=zeros(n,1);		
				for i=1:n
					tinv(i) = 1.0/( delta_b + sig(2*i-1,2*i-1)+sig(2*i,2*i) );
				end
				tinv=diag(tinv);
				
				%A^-1*b -> erg2;
				p_x = tinv*(-f_x(x)' + gx_temp'*(z - sig*g_temp + mu*inv(S)*ones(m,1)));
				p_s = gx_temp*p_x - s + g_temp;
				p_z = -sig*p_s + mu*inv(S)*ones(m,1) - z;

				if Bcomputed==1
					L_b=zeros(m_b);
					D_b=zeros(m_b);
					for i=2:m_b
						for j=1:i-1
							L_b(i,j) = s_b(:,i)'*y_b(:,j);
						end
					end
					for i=1:m_b
						D_b(i,i) = s_b(:,i)'*y_b(:,i);
					end
					
					W_b=[delta_b*s_b(:,1:m_b), y_b(:,1:m_b)];
					M_b=inv( [s_b(:,1:m_b)'*delta_b*s_b(:,1:m_b), L_b ; L_b', -D_b]  );
					
					MWt_b = M_b*W_b';
					
					% A^-1*U -> erg1
					ax1 = -tinv*W_b;
					ax2 = gx_temp*ax1;
					ax3 = -sig*ax2;
					
					% inv(I+VT*erg1)*VT;
					cx = inv(eye(2*m_b) + MWt_b*ax1)*MWt_b;
					
					p_s = p_s - ax2*cx*p_x;
					p_z = p_z - ax3*cx*p_x;
					p_x = p_x - ax1*cx*p_x;
				else
					Bcomputed = 1;
                end
        end
  			
		nu=1.0;
		dphi0 = merit_alpha(x, s, nu, p_x, p_s, 0, f_x, g, g_x, mu);
		if dphi0 > 0
			p_x = -p_x;
			p_s = -p_s;
			dphi0 = -dphi0;
		end

  		alpha_s = 1.0;
		alpha_z = 1.0;
		for i=1:m
			if p_s(i)<0
				new_alpha = -tau*s(i)/p_s(i);
				if new_alpha < alpha_s
					alpha_s = new_alpha;
				end
			end
			if p_z(i)<0
				new_alpha = -tau*z(i)/p_z(i);
				if new_alpha < alpha_z
					alpha_z = new_alpha;
				end
			end
		end

		phia = merit(x, s,nu, p_x, p_s, alpha_s, f, g, mu);
		phi0 = merit(x, s,nu, p_x, p_s, 0, f, g, mu);
		while phia > phi0 + 1e-4*alpha_s*dphi0
			alpha_s = min([ alpha_s, (-dphi0*(alpha_s^2))/(2*(phia - phi0 - dphi0*alpha_s))]);
			
			phia = merit(x, s,nu, p_x, p_s, alpha_s, f, g, mu);
		end
 		
		oldx = x;
		
		s = s + alpha_s*p_s;
		z = z + alpha_z*p_z;
		x = x + alpha_s*p_x;
		
		if isnan(norm(x,inf)) || isnan(norm(s,inf)) || isnan(norm(z,inf))
			err=2;
			return;
		end
		
		yk = (f_x(x)'- g_x(x)'*z) - (f_x(oldx)'- g_x(oldx)'*z);
		sk = x - oldx;
		
		if method == 2 && sk'*yk ~= 0.0
			m_b = min([m_b + 1, max_m_b]);
			count_b = count_b + 1;
			if count_b > max_m_b
				count_b = 1;
			end;
			y_b(:,count_b) = yk;
			s_b(:,count_b) = sk;
		end

		innereps = norm([ f_x(x)'-gx_temp'*z ; S*z-mu*ones(m,1); g_temp-s ],inf);
	end
	
	outereps = norm([ f_x(x)'-gx_temp'*z ; S*z; g_temp-s ],inf);
	
	minprod = inf;
	for i=1:m
		if abs(z(i)*s(i)) < minprod
			minprod = s(i)*z(i);
		end
	end
	
	
	sz_avg = abs((s'*z)/m);
	xi = minprod/( sz_avg );
	sigma = 0.05*(1-xi)/xi;
	if sigma > 2
		sigma = 2;
	end
	sigma = 0.1*sigma;
	
	mu = sigma *sz_avg;
	minustau = sigma*0.1;
	
	outersteps = outersteps +1;
end

if steps==maxsteps
	err=1;
end

end

function y = merit(u, s, nu, p_u, p_s, alpha, f, g, mu)

s = s + p_s*alpha;
u = u + p_u*alpha;

m = size(s,1);

eval_g = g(u);
eval_f = f(u);

temp = 0.0;
for i=1:m
	temp = temp + log(s(i));
end

tempg = 0.0;
for i=1:m
	
	tempg = tempg + (eval_g(i) - s(i))^2;
end

y = eval_f - mu*temp + nu*tempg;

end


function y = merit_alpha(x, s, nu, p_x, p_s, alpha, f_x, g, g_x, mu)

s = s + p_s*alpha;
x = x + p_x*alpha;

m = size(s,1);

eval_g = g(x);
eval_gx = g_x(x);
eval_fx = f_x(x);

temp = 0.0;
for i=1:m
	temp = temp + (eval_g(i) - s(i))*eval_gx(i,:);
end
temp = 2*nu*temp;

firstpart = (eval_fx + temp)*p_x;

secondpart = zeros(1, m);

for i=1:m
	secondpart(1,i) = -mu/s(i) - 2*nu*(eval_g(i) -s(i));
end

secondpart = secondpart*p_s;

y = firstpart + secondpart;

end
